@@ -1,3 +1,4 @@
+#![allow(clippy::needless_doctest_main)]
 //! This crate `cpp` provides macros that allow embedding arbitrary C++ code.
 //!
 //! # Usage
@@ -9,7 +10,7 @@
 //! crate
 //! |-- Cargo.toml
 //! |-- src
-//!     |-- lib.rs
+//!     |-- main.rs
 //! |-- build.rs
 //! ```
 //!
@@ -32,18 +33,16 @@
 //!
 //! ```no_run
 //! extern crate cpp_build;
-//!
 //! fn main() {
-//!     cpp_build::build("src/lib.rs");
+//!     cpp_build::build("src/main.rs");
 //! }
 //! ```
 //!
-//! #### lib.rs
+//! #### main.rs
 //!
 //! ```ignore
 //! # // tested in test/src/examples.rs
-//! #[macro_use]
-//! extern crate cpp;
+//! use cpp::cpp;
 //!
 //! cpp!{{
 //!     #include <iostream>
@@ -71,7 +70,7 @@
 //!
 //! Behind the scene, it uses the `cc` crate.
 //!
-//! ## Using external liraries
+//! ## Using external libraries
 //!
 //! Most likely you will want to link against external libraries. You need to tell cpp_build
 //! about the include path and other flags via `cpp_build::Config` and you need to let cargo
@@ -104,6 +103,16 @@
 //! module into the parsing process. Please note that the latter is only supported
 //! in its simplest form: straight-forward `feature = "..."` without any
 //! additional conditions, `cfg!` macros are also not supported at the moment.
+//!
+//! Since the C++ code is included within a rust file, the C++ code must obey both
+//! the Rust and the C++ lexing rules. For example, Rust supports nested block comments
+//! (`/* ... /* ... */ ... */`) while C++ does not, so nested comments not be used in the
+//! `cpp!` macro. Also the Rust lexer will not understand the C++ raw literal, nor all
+//! the C++ escape sequences within literal, so only string literals that are both valid
+//! in Rust and in C++ should be used. The same applies for group separators in numbers.
+//! Be careful to properly use `#if` / `#else` / `#endif`, and not have unbalanced delimiters.
+
+#![no_std]
 
 #[macro_use]
 #[allow(unused_imports)]
@@ -118,17 +127,17 @@ pub use cpp_macros::*;
 #[macro_export]
 macro_rules! __cpp_internal {
     (@find_rust_macro [$($a:tt)*] rust!($($rust_body:tt)*) $($rest:tt)*) => {
-        __cpp_internal!{ @expand_rust_macro [$($a)*] $($rust_body)* }
-        __cpp_internal!{ @find_rust_macro [$($a)*] $($rest)* }
+        $crate::__cpp_internal!{ @expand_rust_macro [$($a)*] $($rust_body)* }
+        $crate::__cpp_internal!{ @find_rust_macro [$($a)*] $($rest)* }
     };
     (@find_rust_macro [$($a:tt)*] ( $($in:tt)* ) $($rest:tt)* ) =>
-        { __cpp_internal!{ @find_rust_macro [$($a)*] $($in)* $($rest)* }  };
+        { $crate::__cpp_internal!{ @find_rust_macro [$($a)*] $($in)* $($rest)* }  };
     (@find_rust_macro [$($a:tt)*] [ $($in:tt)* ] $($rest:tt)* ) =>
-        { __cpp_internal!{ @find_rust_macro [$($a)*] $($in)* $($rest)* }  };
+        { $crate::__cpp_internal!{ @find_rust_macro [$($a)*] $($in)* $($rest)* }  };
     (@find_rust_macro [$($a:tt)*] { $($in:tt)* } $($rest:tt)* ) =>
-        { __cpp_internal!{ @find_rust_macro [$($a)*] $($in)* $($rest)* }  };
+        { $crate::__cpp_internal!{ @find_rust_macro [$($a)*] $($in)* $($rest)* }  };
     (@find_rust_macro [$($a:tt)*] $t:tt $($rest:tt)*) =>
-        { __cpp_internal!{ @find_rust_macro [$($a)*] $($rest)* } };
+        { $crate::__cpp_internal!{ @find_rust_macro [$($a)*] $($rest)* } };
     (@find_rust_macro [$($a:tt)*]) => {};
 
     (@expand_rust_macro [$($a:tt)*] $i:ident [$($an:ident : $at:ty as $ac:tt),*] {$($body:tt)*}) => {
@@ -140,7 +149,7 @@ macro_rules! __cpp_internal {
         $($a)* unsafe extern "C" fn $i($($an : *const $at),*) {
             $(let $an : $at = unsafe { $an.read() };)*
             (|| { $($body)* })();
-            $(::std::mem::forget($an);)*
+            $(::core::mem::forget($an);)*
 
         }
     };
@@ -156,9 +165,9 @@ macro_rules! __cpp_internal {
             {
                 #[allow(unused_mut)]
                 let mut lambda = || {$($body)*};
-                unsafe { ::std::ptr::write(rt, lambda()) };
+                unsafe { ::core::ptr::write(rt, lambda()) };
             }
-            $(::std::mem::forget($an);)*
+            $(::core::mem::forget($an);)*
             rt
         }
     };
@@ -242,14 +251,14 @@ macro_rules! __cpp_internal {
 #[macro_export]
 macro_rules! cpp {
     // raw text inclusion
-    ({$($body:tt)*}) => { __cpp_internal!{ @find_rust_macro [#[no_mangle] pub] $($body)*} };
+    ({$($body:tt)*}) => { $crate::__cpp_internal!{ @find_rust_macro [#[no_mangle] pub] $($body)*} };
 
     // inline closure
     ([$($captures:tt)*] $($rest:tt)*) => {
         {
-            __cpp_internal!{ @find_rust_macro [] $($rest)*}
+            $crate::__cpp_internal!{ @find_rust_macro [] $($rest)*}
             #[allow(unused)]
-            #[derive(__cpp_internal_closure)]
+            #[derive($crate::__cpp_internal_closure)]
             enum CppClosureInput {
                 Input = (stringify!([$($captures)*] $($rest)*), 0).1
             }
@@ -338,13 +347,13 @@ pub trait CppTrait {
 #[macro_export]
 macro_rules! cpp_class {
     ($(#[$($attrs:tt)*])* unsafe struct $name:ident as $type:expr) => {
-        __cpp_class_internal!{@parse [ $(#[$($attrs)*])* ] [] [unsafe struct $name as $type] }
+        $crate::__cpp_class_internal!{@parse [ $(#[$($attrs)*])* ] [] [unsafe struct $name as $type] }
     };
     ($(#[$($attrs:tt)*])* pub unsafe struct $name:ident as $type:expr) => {
-        __cpp_class_internal!{@parse [ $(#[$($attrs)*])* ] [pub] [unsafe struct $name as $type] }
+        $crate::__cpp_class_internal!{@parse [ $(#[$($attrs)*])* ] [pub] [unsafe struct $name as $type] }
     };
     ($(#[$($attrs:tt)*])* pub($($pub:tt)*) unsafe struct $name:ident as $type:expr) => {
-        __cpp_class_internal!{@parse [ $(#[$($attrs)*])* ] [pub($($pub)*)] [unsafe struct $name as $type] }
+        $crate::__cpp_class_internal!{@parse [ $(#[$($attrs)*])* ] [pub($($pub)*)] [unsafe struct $name as $type] }
     };
 }
 
@@ -353,8 +362,8 @@ macro_rules! cpp_class {
 #[macro_export]
 macro_rules! __cpp_class_internal {
     (@parse [$($attrs:tt)*] [$($vis:tt)*] [unsafe struct $name:ident as $type:expr]) => {
-        __cpp_class_internal!{@parse_attributes [ $($attrs)* ] [] [
-            #[derive(__cpp_internal_class)]
+        $crate::__cpp_class_internal!{@parse_attributes [ $($attrs)* ] [] [
+            #[derive($crate::__cpp_internal_class)]
             #[repr(C)]
             $($vis)* struct $name {
                 _opaque : [<$name as $crate::CppTrait>::BaseType ; <$name as $crate::CppTrait>::ARRAY_SIZE
@@ -365,23 +374,23 @@ macro_rules! __cpp_class_internal {
 
     (@parse_attributes [] [$($attributes:tt)*] [$($result:tt)*]) => ( $($attributes)* $($result)* );
     (@parse_attributes [#[derive($($der:ident),*)] $($tail:tt)* ] [$($attributes:tt)*] [$($result:tt)*] )
-        => (__cpp_class_internal!{@parse_derive [$($der),*] @parse_attributes [$($tail)*] [ $($attributes)* ] [ $($result)* ] } );
+        => ($crate::__cpp_class_internal!{@parse_derive [$($der),*] @parse_attributes [$($tail)*] [ $($attributes)* ] [ $($result)* ] } );
     (@parse_attributes [ #[$m:meta] $($tail:tt)* ] [$($attributes:tt)*] [$($result:tt)*])
-        => (__cpp_class_internal!{@parse_attributes [$($tail)*] [$($attributes)* #[$m] ] [ $($result)* ] } );
+        => ($crate::__cpp_class_internal!{@parse_attributes [$($tail)*] [$($attributes)* #[$m] ] [ $($result)* ] } );
 
-    (@parse_derive [] @parse_attributes $($result:tt)*) => (__cpp_class_internal!{@parse_attributes $($result)*} );
+    (@parse_derive [] @parse_attributes $($result:tt)*) => ($crate::__cpp_class_internal!{@parse_attributes $($result)*} );
     (@parse_derive [PartialEq $(,$tail:ident)*] $($result:tt)*)
-        => ( __cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
+        => ( $crate::__cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
     (@parse_derive [PartialOrd $(,$tail:ident)*] $($result:tt)*)
-        => ( __cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
+        => ( $crate::__cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
     (@parse_derive [Ord $(,$tail:ident)*] $($result:tt)*)
-        => ( __cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
+        => ( $crate::__cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
     (@parse_derive [Default $(,$tail:ident)*] $($result:tt)*)
-        => ( __cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
+        => ( $crate::__cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
     (@parse_derive [Clone $(,$tail:ident)*] $($result:tt)*)
-        => ( __cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
+        => ( $crate::__cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
     (@parse_derive [Copy $(,$tail:ident)*] $($result:tt)*)
-        => ( __cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
+        => ( $crate::__cpp_class_internal!{@parse_derive [$($tail),*] $($result)*} );
     (@parse_derive [$i:ident $(,$tail:ident)*] @parse_attributes [$($attr:tt)*] [$($attributes:tt)*] [$($result:tt)*] )
-        => ( __cpp_class_internal!{@parse_derive [$($tail),*] @parse_attributes [$($attr)*] [$($attributes)* #[derive($i)] ] [ $($result)* ] } );
+        => ( $crate::__cpp_class_internal!{@parse_derive [$($tail),*] @parse_attributes [$($attr)*] [$($attributes)* #[derive($i)] ] [ $($result)* ] } );
 }
